@@ -78,6 +78,7 @@ RESOURCE_ATTRIBUTE_MAP = {
                'validate': {'type:regex': attr.UUID_PATTERN},
                'is_visible': True},
         'name': {'allow_post': True, 'allow_put': True,
+                 'validate': {'type:string': None},
                  'is_visible': True, 'default': ''},
         'admin_state_up': {'allow_post': True, 'allow_put': True,
                            'default': True,
@@ -88,19 +89,23 @@ RESOURCE_ATTRIBUTE_MAP = {
                    'is_visible': True},
         'tenant_id': {'allow_post': True, 'allow_put': False,
                       'required_by_policy': True,
+                      'validate': {'type:string': None},
                       'is_visible': True},
         'external_gateway_info': {'allow_post': True, 'allow_put': True,
                                   'is_visible': True, 'default': None}
     },
     'floatingips': {
         'id': {'allow_post': False, 'allow_put': False,
+               'validate': {'type:uuid': None},
                'is_visible': True},
         'floating_ip_address': {'allow_post': False, 'allow_put': False,
+                                'validate': {'type:ip_address_or_none': None},
                                 'is_visible': True},
         'floating_network_id': {'allow_post': True, 'allow_put': False,
                                 'validate': {'type:regex': attr.UUID_PATTERN},
                                 'is_visible': True},
         'router_id': {'allow_post': False, 'allow_put': False,
+                      'validate': {'type:uuid_or_none': None},
                       'is_visible': True, 'default': None},
         'port_id': {'allow_post': True, 'allow_put': True,
                     'validate': {'type:uuid_or_none': None},
@@ -110,6 +115,7 @@ RESOURCE_ATTRIBUTE_MAP = {
                              'is_visible': True, 'default': None},
         'tenant_id': {'allow_post': True, 'allow_put': False,
                       'required_by_policy': True,
+                      'validate': {'type:string': None},
                       'is_visible': True}
     },
 }
