@@ -40,6 +40,7 @@ from quantum.agent import rpc as agent_rpc
 from quantum.common import config as logging_config
 from quantum.common import constants
 from quantum.common import topics
+from quantum.common import utils as q_utils
 from quantum.openstack.common import cfg
 from quantum.openstack.common import context
 from quantum.openstack.common import rpc
@@ -427,8 +428,10 @@ class LinuxBridgeRpcCallbacks():
         port = kwargs.get('port')
         if port['admin_state_up']:
             vlan_id = kwargs.get('vlan_id')
+            physical_network = kwargs.get('physical_network')
             # create the networking for the port
             self.linux_br.add_interface(port['network_id'],
+                                        physical_network,
                                         vlan_id,
                                         port['id'])
         else:
@@ -636,7 +639,8 @@ class LinuxBridgeQuantumAgentRPC:
             if devices:
                 mac = utils.get_interface_mac(devices[0].name)
             else:
-                LOG.error("Unable to obtain MAC of any device for agent_id")
+                LOG.error("Unable to obtain MAC address for unique ID. "
+                          "Agent terminated!")
                 exit(1)
         self.agent_id = '%s%s' % ('lb', (mac.replace(":", "")))
         LOG.info("RPC agent_id: %s" % self.agent_id)
@@ -792,17 +796,14 @@ def main():
     # (TODO) gary - swap with common logging
     logging_config.setup_logging(cfg.CONF)
 
-    interface_mappings = {}
-    for mapping in cfg.CONF.LINUX_BRIDGE.physical_interface_mappings:
-        try:
-            physical_network, physical_interface = mapping.split(':')
-            interface_mappings[physical_network] = physical_interface
-            LOG.debug("physical network %s mapped to physical interface %s" %
-                      (physical_network, physical_interface))
-        except ValueError as ex:
-            LOG.error("Invalid physical interface mapping: \'%s\' - %s" %
-                      (mapping, ex))
-            sys.exit(1)
+    try:
+        interface_mappings = q_utils.parse_mappings(
+            cfg.CONF.LINUX_BRIDGE.physical_interface_mappings)
+    except ValueError as e:
+        LOG.error(_("Parsing physical_interface_mappings failed: %s."
+                    " Agent terminated!"), e)
+        sys.exit(1)
+    LOG.info(_("Interface mappings: %s") % interface_mappings)
 
     polling_interval = cfg.CONF.AGENT.polling_interval
     reconnect_interval = cfg.CONF.DATABASE.reconnect_interval
@@ -823,5 +824,6 @@ def main():
     plugin.daemon_loop()
     sys.exit(0)
 
+
 if __name__ == "__main__":
     main()
