@@ -31,6 +31,7 @@ from webob import exc
 
 from quantum.api.v2 import attributes
 from quantum.common import config
+from quantum.common import exceptions as q_exc
 from quantum.common.test_lib import test_config
 from quantum.common import utils
 from quantum import context
@@ -425,6 +426,46 @@ class L3NatDBTestCase(test_db_plugin.QuantumDbPluginV2TestCase):
                                                              s['subnet']['id'],
                                                              None)
 
+    def test_router_add_interface_subnet_with_port_from_other_tenant(self):
+        tenant_id = _uuid()
+        other_tenant_id = _uuid()
+        tenant_context = context.Context(user_id=None, tenant_id=tenant_id)
+        admin_context = context.get_admin_context()
+        with mock.patch('quantum.context.Context') as ctx:
+            ctx.return_value = admin_context
+            with contextlib.nested(
+                self.router(tenant_id=tenant_id),
+                self.network(tenant_id=tenant_id),
+                self.network(tenant_id=other_tenant_id)) as (r, n1, n2):
+                with contextlib.nested(
+                    self.subnet(network=n1, cidr='10.0.0.0/24'),
+                    self.subnet(network=n2, cidr='10.1.0.0/24')) as (s1, s2):
+                        ctx.return_value = admin_context
+                        body = self._router_interface_action(
+                            'add',
+                            r['router']['id'],
+                            s2['subnet']['id'],
+                            None)
+                        self.assertTrue('port_id' in body)
+                        ctx.return_value = tenant_context
+                        self._router_interface_action(
+                            'add',
+                            r['router']['id'],
+                            s1['subnet']['id'],
+                            None)
+                        self.assertTrue('port_id' in body)
+                        self._router_interface_action(
+                            'remove',
+                            r['router']['id'],
+                            s1['subnet']['id'],
+                            None)
+                        ctx.return_value = admin_context
+                        body = self._router_interface_action(
+                            'remove',
+                            r['router']['id'],
+                            s2['subnet']['id'],
+                            None)
+
     def test_router_add_interface_port(self):
         with self.router() as r:
             with self.port(no_delete=True) as p:
@@ -520,6 +561,40 @@ class L3NatDBTestCase(test_db_plugin.QuantumDbPluginV2TestCase):
                                                       None,
                                                       p1['port']['id'])
 
+    def test_router_add_gateway_dup_subnet1(self):
+        with self.router() as r:
+            with self.subnet() as s:
+                body = self._router_interface_action('add',
+                                                     r['router']['id'],
+                                                     s['subnet']['id'],
+                                                     None)
+                self._set_net_external(s['subnet']['network_id'])
+                self._add_external_gateway_to_router(
+                    r['router']['id'],
+                    s['subnet']['network_id'],
+                    expected_code=exc.HTTPBadRequest.code)
+                body = self._router_interface_action('remove',
+                                                     r['router']['id'],
+                                                     s['subnet']['id'],
+                                                     None)
+
+    def test_router_add_gateway_dup_subnet2(self):
+        with self.router() as r:
+            with self.subnet() as s:
+                self._set_net_external(s['subnet']['network_id'])
+                self._add_external_gateway_to_router(
+                    r['router']['id'],
+                    s['subnet']['network_id'])
+                self._router_interface_action('add',
+                                              r['router']['id'],
+                                              s['subnet']['id'],
+                                              None,
+                                              expected_code=exc.
+                                              HTTPBadRequest.code)
+                self._remove_external_gateway_from_router(
+                    r['router']['id'],
+                    s['subnet']['network_id'])
+
     def test_router_add_interface_overlapped_cidr(self):
         with self.router() as r:
             with self.subnet(cidr='10.0.1.0/24') as s1:
@@ -557,6 +632,21 @@ class L3NatDBTestCase(test_db_plugin.QuantumDbPluginV2TestCase):
                                                  expected_code=exc.
                                                  HTTPBadRequest.code)
 
+    def test_create_router_with_gwinfo(self):
+        with self.subnet() as s:
+            self._set_net_external(s['subnet']['network_id'])
+            data = {'router': {'tenant_id': _uuid()}}
+            data['router']['name'] = 'router1'
+            data['router']['external_gateway_info'] = {
+                'network_id': s['subnet']['network_id']}
+            router_req = self.new_create_request('routers', data, 'json')
+            res = router_req.get_response(self.ext_api)
+            router = self.deserialize('json', res)
+            self.assertEquals(
+                s['subnet']['network_id'],
+                router['router']['external_gateway_info']['network_id'])
+            self._delete('routers', router['router']['id'])
+
     def test_router_add_gateway(self):
         with self.router() as r:
             with self.subnet() as s:
@@ -574,6 +664,30 @@ class L3NatDBTestCase(test_db_plugin.QuantumDbPluginV2TestCase):
                 gw_info = body['router']['external_gateway_info']
                 self.assertEquals(gw_info, None)
 
+    def test_router_update_gateway(self):
+        with self.router() as r:
+            with self.subnet() as s1:
+                with self.subnet() as s2:
+                    self._set_net_external(s1['subnet']['network_id'])
+                    self._add_external_gateway_to_router(
+                        r['router']['id'],
+                        s1['subnet']['network_id'])
+                    body = self._show('routers', r['router']['id'])
+                    net_id = (body['router']
+                              ['external_gateway_info']['network_id'])
+                    self.assertEquals(net_id, s1['subnet']['network_id'])
+                    self._set_net_external(s2['subnet']['network_id'])
+                    self._add_external_gateway_to_router(
+                        r['router']['id'],
+                        s2['subnet']['network_id'])
+                    body = self._show('routers', r['router']['id'])
+                    net_id = (body['router']
+                              ['external_gateway_info']['network_id'])
+                    self.assertEquals(net_id, s2['subnet']['network_id'])
+                    self._remove_external_gateway_from_router(
+                        r['router']['id'],
+                        s2['subnet']['network_id'])
+
     def test_router_add_gateway_invalid_network(self):
         with self.router() as r:
             self._add_external_gateway_to_router(
@@ -683,7 +797,7 @@ class L3NatDBTestCase(test_db_plugin.QuantumDbPluginV2TestCase):
 
     @contextlib.contextmanager
     def floatingip_with_assoc(self, port_id=None, fmt='json'):
-        with self.subnet() as public_sub:
+        with self.subnet(cidr='11.0.0.0/24') as public_sub:
             self._set_net_external(public_sub['subnet']['network_id'])
             with self.port() as private_port:
                 with self.router() as r:
@@ -714,7 +828,7 @@ class L3NatDBTestCase(test_db_plugin.QuantumDbPluginV2TestCase):
 
     @contextlib.contextmanager
     def floatingip_no_assoc(self, private_sub, fmt='json'):
-        with self.subnet() as public_sub:
+        with self.subnet(cidr='12.0.0.0/24') as public_sub:
             self._set_net_external(public_sub['subnet']['network_id'])
             with self.router() as r:
                 self._add_external_gateway_to_router(
@@ -749,6 +863,44 @@ class L3NatDBTestCase(test_db_plugin.QuantumDbPluginV2TestCase):
         self._show('floatingips', fip['floatingip']['id'],
                    expected_code=exc.HTTPNotFound.code)
 
+    def test_floatingip_with_assoc_fails(self):
+        fmt = 'json'
+        with self.subnet(cidr='200.0.0.1/24') as public_sub:
+            self._set_net_external(public_sub['subnet']['network_id'])
+            with self.port() as private_port:
+                with self.router() as r:
+                    sid = private_port['port']['fixed_ips'][0]['subnet_id']
+                    private_sub = {'subnet': {'id': sid}}
+                    self._add_external_gateway_to_router(
+                        r['router']['id'],
+                        public_sub['subnet']['network_id'])
+                    self._router_interface_action('add', r['router']['id'],
+                                                  private_sub['subnet']['id'],
+                                                  None)
+                    PLUGIN_CLASS = 'quantum.db.l3_db.L3_NAT_db_mixin'
+                    METHOD = PLUGIN_CLASS + '._update_fip_assoc'
+                    with mock.patch(METHOD) as pl:
+                        pl.side_effect = q_exc.BadRequest(
+                            resource='floatingip',
+                            msg='fake_error')
+                        res = self._create_floatingip(
+                            fmt,
+                            public_sub['subnet']['network_id'],
+                            port_id=private_port['port']['id'])
+                        self.assertEqual(res.status_int, 400)
+
+                    for p in self._list('ports')['ports']:
+                        if p['device_owner'] == 'network:floatingip':
+                            self.fail('garbage port is not deleted')
+
+                    self._remove_external_gateway_from_router(
+                        r['router']['id'],
+                        public_sub['subnet']['network_id'])
+                    self._router_interface_action('remove',
+                                                  r['router']['id'],
+                                                  private_sub['subnet']['id'],
+                                                  None)
+
     def test_floatingip_update(self):
         with self.port() as p:
             private_sub = {'subnet': {'id':
@@ -794,16 +946,13 @@ class L3NatDBTestCase(test_db_plugin.QuantumDbPluginV2TestCase):
                 self.assertEquals(body['floatingip']['fixed_ip_address'], None)
                 self.assertEquals(body['floatingip']['router_id'], None)
 
-    def test_double_floating_assoc(self):
+    def test_two_fips_one_port_invalid_return_409(self):
         with self.floatingip_with_assoc() as fip1:
-            with self.subnet() as s:
-                with self.floatingip_no_assoc(s) as fip2:
-                    port_id = fip1['floatingip']['port_id']
-                    body = self._update('floatingips',
-                                        fip2['floatingip']['id'],
-                                        {'floatingip':
-                                         {'port_id': port_id}},
-                                        expected_code=exc.HTTPConflict.code)
+            res = self._create_floatingip(
+                'json',
+                fip1['floatingip']['floating_network_id'],
+                fip1['floatingip']['port_id'])
+            self.assertEqual(res.status_int, exc.HTTPConflict.code)
 
     def test_floating_ip_direct_port_delete_returns_409(self):
         found = False
